;(function ($) {
    /**
     * 下拉(复选)树控件
     * @class FR.NewComboBoxTreeEditor
     * @extends FR.TreeComboBoxEditor
     */
    FR.NewComboBoxTreeEditor = FR.extend(FR.TreeComboBoxEditor, /**@class FR.TreeComboBoxEditor*/{

        _init: function () {
            FR.TreeComboBoxEditor.superclass._init.apply(this, arguments);
            this.$view.attr("id", this.options.location);
        },

        _createTreeView: function (treeConfig) {
            treeConfig.returnAllSelectNodes = true;
            this.$view.treeview(treeConfig);
        },
        _onEnterPressed: function (node, _onEnterPressed) {
            if (!this.options.mutiSelection) {
                this.text = this.$view.getTCPText();
                this.value = this.$view.getTCPValue();
                var len = this.text.length;
                this.editComp.val(len === 0 ? "" : this.text[len - 1]);
                this.collapse();
            } else if (!this.options.selectLeafOnly) {
                var tValue = node.getValuePath();
                //bug:71763 树控件的text应该统一成二维数组
                var tText = FR.parseString2Array4Tree(node.getShowValue(), this);
                if (node.getcheckstate() === 1) { // 点击选中
                    this.value.push(tValue);
                    this.text.push(tText);
                } else if (node.getcheckstate() === 0) { // 点击取消选中
                    for (var i = 0; i < this.value.length; i++) {
                        if (FR.equals(tValue, this.value[i])) {
                            this.value.remove(this.value[i]);
                            break;
                        }
                    }
                    this.text.removeWeakEqual(tText);
                }
                this._dealWithEditComp();
            } else {
                //多选且返回叶子结点
                var children = node.getAllChildren();
                children = children.concat(this.getBubbleParents(node), this.getCascadeParents(node));
                var textArr = [];
                var valueArr = [];
                if (node.getcheckstate() === 1) {
                    for (var i = 0; i < children.length; i++) {
                        if (this.value.weakIndexOf(children[i].getValuePath()) !== -1) {//选中的时候要过滤掉已经是选中状态的REPORT-9931
                            continue;
                        }
                        valueArr.push(children[i].getValuePath());
                        textArr.push(children[i].getTextPath());
                    }
                    this.text = this.text.concat(textArr);
                    this.value = this.value.concat(valueArr);
                } else if (node.getcheckstate() === 0) {
                    for (var i = 0; i < children.length; i++) {
                        valueArr.push(children[i].getValuePath());
                        textArr.push(children[i].getTextPath());
                    }
                    this.text = this._difference(this.text, textArr);
                    this.value = this._difference(this.value, valueArr);
                }
                this._dealWithEditComp(this.text);
            }
            this.fireEvent(FR.Events.CLICK);
            if (_onEnterPressed !== false) {
                this.fireEvent(FR.Events.AFTEREDIT);
            }
        },
        /**
         * 选中一个节点的时候,遍历所有的父节点,找到所有选中状态的父节点
         * @param node
         * @returns {*}
         */
        getBubbleParents: function (node) {
            var isSelect = node.getcheckstate() === 1;
            var select_array = node.hasChild() ? [node] : [];
            var cancel_array = node.hasChild() ? [node] : [];
            var p = node.getParent();
            while (p) {
                if (isSelect && p.getcheckstate() === 1) {
                    select_array.push(p);
                } else if (!isSelect && p.getcheckstate() !== 1) {
                    cancel_array.push(p);
                }
                p = p.getParent();
            }
            return isSelect ? select_array : cancel_array;
        },
        /**
         * 选中一个节点的时候,获取所有子节点的目录节点，不包括自己
         * @param node
         * @returns {*}
         */
        getCascadeParents: function (node) {
            var children = node.getChildren();
            var result = [];
            if (children.length == 0) {
                return [];
            }
            for (var i = 0; i < children.length; i++) {
                var child = children[i];
                if (child.getChildren().length != 0) {
                    result.push(child);
                    result = result.concat(this.getCascadeParents(child));
                }
            }
            return result;
        }
    });
    $.shortcut('newcomboboxtree', FR.NewComboBoxTreeEditor);
})(jQuery);

!(function () {
    try {
        var provider = BI.Providers.getProvider("report.controller.provider");
    } catch (e) {
        return ;
    }

    var ImplClass = provider.getImpl();

    var Widget = BI.inherit(ImplClass, {
        mounted: function() {
            var self = this;

            this.widget = FR.createWidget(BI.extend({}, this.options, {
                type: 'newcomboboxtree',
                listeners: this.options.xlisteners,
                renderEl: this.element[0]
            }));

            this.widget.on('click', function () {
                self.fireEvent('EVENT_CHANGE');
            });
        },

        initData: function() { return (this.widget && this.widget.initData.apply(this.widget, arguments)) },

        focusDown: function() { return (this.widget && this.widget.focusDown.apply(this.widget, arguments)) },

        focusUp: function() { return (this.widget && this.widget.focusUp.apply(this.widget, arguments)) },

        setOptions: function() { return (this.widget && this.widget.setOptions.apply(this.widget, arguments)) },

        setTraversal: function() { return (this.widget && this.widget.setTraversal.apply(this.widget, arguments)) },

        checkViewScroll: function() { return (this.widget && this.widget.checkViewScroll.apply(this.widget, arguments)) },

        initFilterData: function() { return (this.widget && this.widget.initFilterData.apply(this.widget, arguments)) },

        dealWithDisplayValue: function() { return (this.widget && this.widget.dealWithDisplayValue.apply(this.widget, arguments)) },

        getValue: function() { return (this.widget && this.widget.getValue.apply(this.widget, arguments)) },

        getEditValue: function() { return (this.widget && this.widget.getEditValue.apply(this.widget, arguments)) },

        getFullPathValue: function() { return (this.widget && this.widget.getFullPathValue.apply(this.widget, arguments)) },

        recoveryValue: function() { return (this.widget && this.widget.recoveryValue.apply(this.widget, arguments)) },

        getText: function() { return (this.widget && this.widget.getText.apply(this.widget, arguments)) },

        fixViewWidth: function() { return (this.widget && this.widget.fixViewWidth.apply(this.widget, arguments)) },

        isComboBoxTree: function() { return (this.widget && this.widget.isComboBoxTree.apply(this.widget, arguments)) },

        reset: function() { return (this.widget && this.widget.reset.apply(this.widget, arguments)) },

        getValueNodes: function() { return (this.widget && this.widget.getValueNodes.apply(this.widget, arguments)) },

        getNode: function() { return (this.widget && this.widget.getNode.apply(this.widget, arguments)) },

        getSelectedNodes: function() { return (this.widget && this.widget.getSelectedNodes.apply(this.widget, arguments)) },

        getNodeByID: function() { return (this.widget && this.widget.getNodeByID.apply(this.widget, arguments)) },

        isValueInList: function() { return (this.widget && this.widget.isValueInList.apply(this.widget, arguments)) },

        isValidate: function() { return (this.widget && this.widget.isValidate.apply(this.widget, arguments)) },

        preGetValidation: function() { return { result: this.isValidate.apply(this, arguments) } },

        postGetValidation: function() { return { result: this.isValidate.apply(this, arguments) } },

        getBubbleParents: function() { return (this.widget && this.widget.getBubbleParents.apply(this.widget, arguments)) },

        getCascadeParents: function() { return (this.widget && this.widget.getCascadeParents.apply(this.widget, arguments)) },

        setInteractValue: function(v) {
            this.widget && this.widget.setValue.apply(this.widget, [v ? v.value : ""]);
        },

        setControllerValue: function() { this.widget && this.widget.setValue.apply(this.widget, arguments) },

        resize: function(width, height) {
            this.widget && this.widget.doResize({
                width: width,
                height: height
            });
        }
    });

    Widget.xtype = "report.newcomboboxtree";

    BI.shortcut(Widget.xtype, Widget);

    provider.set('newcomboboxtree', Widget.xtype);
})();